/*******************************************************************************
*                      
*  Filename:      K:\keyfob\tony\winfob\Csp2.cpp
*
*  Copyright(c) Symbol Technologies Inc., 2001
*  
*  Description:     This file provides the user API interface
*                   functions for Symbol's CS1504 Consumer Scanning 
*                   Products as Dynamic Link Library. When 
*                   compiled into a DLL, the user can access 
*                   all of the functions available for the 
*                   Symbol CS1504.
*
*  Author:          Tony Russo
*
*  Creation Date:   4/04/2001
*
*  Derived From:    New File
*
*  Edit History:
*   $Log:   U:\keyfob\winfob\source\c\csp2.cpv  $
* 
*    Rev 1.0   05 Apr 2001 09:08:10   RUSSOA
* Initial revision.
*
*******************************************************************************/
                    
/*******************************************************************************
* Include Files
*******************************************************************************/
#ifndef WIN32
    #define WIN32
#endif

#pragma hdrstop

/** include files **/
#include "windows.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "winver.h"                 // DLL Version information
#include "winbase.h"                // File I/O for debug
#include "CSP2.h"                 // function proptotypes for DLL
#include "CRCCalc.h"				// CRC function prototypes
#include <sys/timeb.h>
#include <time.h>

#ifdef _DEBUG
    #include <time.h>               // time stamps for debug files
    
    #define new DEBUG_NEW
    #undef THIS_FILE
    static char THIS_FILE[] = __FILE__;
#endif

/*******************************************************************************
* Defines
*******************************************************************************/ 
/** local definitions **/
#define STX                         ((char)0x02)    // <stx> character
#define	CS1504_PWR_UP_TIME			400             // delay from pwr down to up                       
#define POLL_INTERVAL				500				// 500 ms (.5 second) interval
// Command byte values...
#define INTERROGATE                 ((char)0x01)
#define CLEAR_BAR_CODES             ((char)0x02)
#define DOWNLOAD_PARAMETERS         ((char)0x03)
#define SET_DEFAULTS				((char)0x04)
#define POWER_DOWN                  ((char)0x05)
#define RESERVED_CODE_6             ((char)0x06)
#define UPLOAD                      ((char)0x07)
#define UPLOAD_PARAMETERS           ((char)0x08)
#define SET_TIME					((char)0x09)
#define GET_TIME					((char)0x0a)

#define Param_ASCII_Mode			((char)0x4f)
#define Param_Store_RTC				((char)0x23)

// CSP device status value...
#define NO_ERROR_ENCOUNTERED        ((long)-6)

#define MAXTIME                     ((long)  5)    // 50 millisecond ticks, ~ 1 second
#define MAXSIZE                     ((long)16384)   // maximum size of retrieved barcodes
#define RX_QUE_SIZE                 ((long)1024)    // communications RX buffer setting
#define TX_QUE_SIZE                 ((long) 512)    // communications Tx buffer setting

#define HS_SET                      ((long) 1)
#define HS_CLEAR                    ((long) 0)

#define MAX_XMIT_SIZE				((long) 60)

/*******************************************************************************
* Public Variables
*******************************************************************************/ 
/* default settings */
long	nDelayCount					= MAXTIME;		// used for interrogate timeouts
long    nCspActivePort              = -1;           // no comm port selected
long    nCspDeviceStatus            = -1;           // no current Device status
long    nCspProtocolVersion         = -1;           // no protocol version available
long    nCspSystemStatus            = -1;           // no system status available

long    nCspStoredBarcodes          =  0;           // number of stored barcode strings
long    nCspRetryCount              =  5;           // default number of interrogation retries

long	nCspASCII					= -1;			// CS1504 ASCII mode status
long	nCspRTC						= -1;			// CS1504 RTC Storage Status

/*******************************************************************************
* Local Variables
*******************************************************************************/ 
/** private data **/
#ifdef _DEBUG
    BOOL    bCspDebugMode           = TRUE;			// flag indicating debug mode is active
    HANDLE  hCspDebug;                              // handle to debug text file
    char    szCspDebug[80];                         // Debug text
    DWORD   nCspDebugLength;                        // number of characters in debug text
    char    *lpszCspDebug;                          // pointer to a debug string
    char    szDebugTime[10];                        // Time stamp string
#endif

/** CallBack Thread **/
DWORD dwThreadId, dwThrdParam = 1;
HANDLE hThread; 
FARPROC csp2PollCallBack;
bool Polling = false;
struct _timeb PwrDwnTime;

/*******************************************************************************
* Public Function Prototypes 
*******************************************************************************/ 
DCB		dcb,OriginalDcb;			   // device control blocks for Comm ports
HANDLE	hCom = INVALID_HANDLE_VALUE;   // handle for comm port
DWORD	dwError;					   // errors from Win comm API
BOOL	fSuccess;					   // API call results

HMODULE hModuleThis;				   // handle to get version info

static long    nCspVolume;             // Device volume setting
static char    szCspSwVersion[9];      // Device software version
static char    szCspDeviceId[9];       // Device ID
static char    szCspBarData[MAXSIZE];  // uploaded barcode storage
static char    aByteBuffer[MAXSIZE];   // temp storage of CSP device response
BYTEWORD CRCtest;					   // temp var for CRC checks

/*******************************************************************************
* Local Functions Prototypes
*******************************************************************************/ 
/** private functions **/
static long csp2Getc(void);
static char csp2LrcCheck(char aLrcBytes[], long nMaxLength);
static void csp2InitParms(void);
static long csp2SendCommand (char *aCommand, long nMaxLength);
static long GetCTS(void);
static long GetParam( long nParam, char szString[], long nMaxLength );


/*********************************************************************                      
* Synopsis:      BOOL WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,
*                                    LPVOID lpvReserved)
*
* Description:   DllMain is used to save the instance handle to this DLL 
*                    - For version use   
*
* Parameters:       
*
* Return Value:     
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/

BOOL WINAPI DllMain(  HINSTANCE hinstDLL,  // handle to DLL module
		DWORD fdwReason,              // reason for calling function
		LPVOID lpvReserved)          // reserved);
{
	switch (fdwReason)		// check to see if we need to kill a thread
	{
        case DLL_PROCESS_DETACH:  
			if (Polling) csp2StopPolling();
			break;
        default:
			break;      
	} 

	hModuleThis=hinstDLL;
	return(TRUE);
}

/*********************************************************************                      
* Synopsis:       DWORD WINAPI ThreadFunc(  LPVOID lpParameter )
*
* Description:    This is the function associated with the polling thread.
*				  It monitors the incoming handshake and trips the callback 
*				  if it sees a CS1504
*
* Parameters:     lpParameter is passed from the spawning thread, and is
*				  currently unused.
*
* Return Value:   STATUS_OK regardless.
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
DWORD WINAPI ThreadFunc(  LPVOID lpParameter )
{
	while (Polling) 
	{
		if (GetCTS()==HS_SET) csp2PollCallBack();
		Sleep(POLL_INTERVAL);
	}
	return(STATUS_OK);
}


/*********************************************************************                      
* Synopsis:       NoMangle long DLL_IMPORT_EXPORT csp2StopPolling(void)
*
* Description:    Cancels the seperate polling thread.
*
* Parameters:     None.
*
* Return Value:   STATUS_OK if ok.
*				  GENERAL_ERROR we had to unnaturally terminate the thread
*				  Positive value is the error if the thread did not return
*				  a status.
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
// Stop polling the 1504
NoMangle long DLL_IMPORT_EXPORT csp2StopPolling(void)
{
	int i;
	DWORD exitcode;
	
	Polling=false;
	for (i=0;i<11;i++)
	{
		if (GetExitCodeThread(hThread,&exitcode))
		{
			if (exitcode==STILL_ACTIVE) Sleep(50);
			else 
			{
				CloseHandle(hThread);
				return(STATUS_OK);
			}
		}
		else // something bad happened
		{
			exitcode=GetLastError();
			TerminateThread(hThread,NULL);
			CloseHandle(hThread);
			return(exitcode);
		}
	}
	// Thread still running, but should have ended. Terminate it.
	TerminateThread(hThread,NULL);
	CloseHandle(hThread);
	return(GENERAL_ERROR);
}
					
/*********************************************************************                      
* Synopsis:       NoMangle long DLL_IMPORT_EXPORT csp2StartPolling
*					(FARPROC csp2CallBack)
*
* Description:    This function spawns a new thread to poll for a 
*				  CS1504. The callback address allows the thread to
*                 inform the caller that a device has been detected.
*
* Parameters:     FARPROC csp2CallBack is the address of the callers 
*				  callback function  
*
* Return Value:   STATUS_OK if ok.
*				  GENERAL_ERROR if already polling
*				  BAD_PARAM if invalid callback provided
*				  SETUP_ERROR if problem occurred spawning the thread
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/

// Set up to poll the 1504
NoMangle long DLL_IMPORT_EXPORT csp2StartPolling(FARPROC csp2CallBack)
{
long retval;

	retval=STATUS_OK;
	if (Polling) return(GENERAL_ERROR); // already polling
	if (NULL==csp2CallBack) return(BAD_PARAM);
	csp2PollCallBack=csp2CallBack;
	Polling=true;
// Create a thread to poll for CS1504	
	hThread = CreateThread( 
    NULL,                        // no security attributes 
    0,                           // use default stack size  
    ThreadFunc,                  // thread function 
    &dwThrdParam,                // argument to thread function 
    0,                           // use default creation flags 
    &dwThreadId);                // returns the thread identifier  
// Check the return value for success. 
    if (hThread == NULL)
	{
		retval=SETUP_ERROR;
		Polling=false;
		CloseHandle( hThread );
	}
	return(retval);
}


/*********************************************************************                      
* Synopsis:       static long csp2Getc(void)
*
* Description:    This function retrieves a character from the 
*                 active serial port. If the serial port does not
*                 have a character within the MAXTIME period, the
*                 function returns with a COMMUNICATIONS_ERROR.  
*
* Parameters:     None  
*
* Return Value:   character read (zero or positive value)  
*                 COMMUNICATIONS_ERROR if no character was read
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
static long csp2Getc(void)
{
    long i;
    char  chRead[32];
	DWORD BytesRead;

    // make sure that a port has been selected...   
    if (nCspActivePort >= COM1)
        {
        // a port is selected, can we retrieve any data from it?
        for (i = 0; i < nDelayCount; i++)
            {
            // try to read a character...
			BytesRead=0;
			ReadFile(hCom,&chRead,1,&BytesRead,NULL);
            if (BytesRead)
                {
                #ifdef _DEBUG
                    if (bCspDebugMode)
                        {
                        nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                    "R[0x%02X] ", 
                                                    (unsigned char)chRead[0]);
                        WriteFile(  hCspDebug,          // handle to debug text file
                                    szCspDebug,         // buffer of data to write to file
                                    nCspDebugLength,    // number of bytes to write
                                    &nCspDebugLength,   // number of bytes written
                                    0);                 // no overlapped structure present
                        }
                #endif
                
                return((long) chRead[0]);
                }

            // let other tasks run while we wait for a character...
            Sleep(50);
            }
        }

    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "[device time out]\r\n");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

   // if we make it this far, no character was read. So...  
   return( COMMUNICATIONS_ERROR );
}

/*********************************************************************                      
* Synopsis:       static char csp2LrcCheck(char aLrcBytes[], long nMaxLength)
*
* Description:    THIS ROUTINE IS KEPT FOR POTENTIAL FUTURE USE WITH CS2000 DEVICES
*                 Computes the Longitudinal Redundancy Check on a
*                 string of specified length.
*
* Parameters:     aLrcBytes[] contains the bytes used for computing the LRC
*                 nMaxLength determines how many bytes of aLrcBytes[] are used
*                 to perform LRC checking.
*
* Return Value:    The calculated LRC is returned. 
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
static char csp2LrcCheck(char aLrcBytes[], long nMaxLength)
{
    long i;
    unsigned char chLrc = 0;        // initialize LRC value

    // compute the LRC for the entire string...
    for (i = 0; i < nMaxLength; i++)
        chLrc ^= aLrcBytes[i];

    return( chLrc );
}

/*********************************************************************                      
* Synopsis:        static void csp2InitParms(void)
*
* Description:     Invalidates all values used by this DLL. Used
*                  to indicate that previously stored data is no
*                  longer valid, possibly because of a new comm
*                  port setting. 
*
* Parameters:      None 
*
* Return Value:    None 
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
static void csp2InitParms(void)
{
    nCspActivePort              = -1;   // no comm port selected
    nCspDeviceStatus            = -1;   // no current Device status
    nCspProtocolVersion         = -1;   // no protocol version available
    nCspSystemStatus            = -1;   // no system status available

	nCspASCII					= -1;	// CS1504 ASCII mode status unavailable
	nCspRTC						= -1;	// CS1504 RTC Storage Status unavailable

    nCspStoredBarcodes          =  0;   // number of stored barcode strings
}

/*********************************************************************                      
* Synopsis:        static long csp2SendCommand (char *aCommand, long nMaxLength)
*
* Description:     This function handles the basic overhead associated
*                  with sending the CSP device a command. Examples of
*                  commands for the CSP device are:
*                      Interrogation
*                      Read Barcodes
*                      Clear Barcodes
*                      Power Down
*                      Get parameter
*                      Set parameter
*
*                  aByteBuffer[0] contains the first data byte read
*                  from the CSP device.
*
* Parameters:      aCommand[] is an array of command bytes to send
*                  to the CSP device.
*                  nMaxLength specifies how many bytes in aCommand[]
*                  are to be sent to the CSP device.
*
* Return Value:    STATUS_OK
*                  COMMUNICATIONS_ERROR
*                  INVALID_COMMAND_NUMBER
*                  COMMAND_LRC_ERROR
*                  RECEIVED_CHARACTER_ERROR
*                  GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
static long csp2SendCommand (char *aCommand, long nMaxLength)
{
	DWORD nBytesWritten;
    
    // make sure we have a valid port selected...
    if (nCspActivePort < COM1)
        return( COMMUNICATIONS_ERROR );

    // delay before sending message
    Sleep ( 200 );

    // clear the receive queue...
    fSuccess=PurgeComm(hCom,PURGE_RXCLEAR);
	if (!fSuccess) return(COMMUNICATIONS_ERROR);

    // send the command string...
	fSuccess=WriteFile(hCom,aCommand,nMaxLength,&nBytesWritten,NULL);
	if (!fSuccess) return(COMMUNICATIONS_ERROR);

	// Get status return
    nCspDeviceStatus  = csp2Getc();
    aByteBuffer[0] = (char) nCspDeviceStatus;

    // convert Device status to CSP status values
    nCspDeviceStatus *= -1;

    // if response not available, reply with error
    if ( nCspDeviceStatus > 0 )
        return( COMMUNICATIONS_ERROR );

    if ( nCspDeviceStatus != NO_ERROR_ENCOUNTERED )
        return ( nCspDeviceStatus );
    else
        return ( STATUS_OK );
}


/*********************************************************************                      
* Synopsis:       long GetCTS(void)
*
* Description:    Get status of the CTS line
*
* Parameters:     None  
*
* Return Value:   <0 for error, 0 for clear, 1 for set
  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
static long GetCTS(void) 
{
	DWORD ModemStat;

	if (hCom == INVALID_HANDLE_VALUE) return(COMMUNICATIONS_ERROR);

	fSuccess=GetCommModemStatus(hCom,&ModemStat);
	if (!fSuccess) return(COMMUNICATIONS_ERROR);
	if (ModemStat&MS_CTS_ON) return(HS_SET);
	else return(HS_CLEAR);

}

/*********************************************************************                      
* Synopsis:       long GetDTR(void)
*
* Description:    Get status of DTR output line
*
* Parameters:     None  
*
* Return Value:   <0 for error, 0 for clear, 1 for set  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
long GetDTR(void)
{
	if (hCom == INVALID_HANDLE_VALUE) return(COMMUNICATIONS_ERROR);

	fSuccess = GetCommState(hCom, &dcb);
	
	if (!fSuccess) return(COMMUNICATIONS_ERROR);

	if (dcb.fDtrControl=DTR_CONTROL_ENABLE) return(HS_SET);
	else return(HS_CLEAR);

}

/*********************************************************************                      
* Synopsis:       long SetDTR(long nOnOff)
*
* Description:    Set the DTR Output line  
*
* Parameters:       
*
* Return Value:   0 for success, <0 for error  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
long SetDTR(long nOnOff)
{
	if (hCom == INVALID_HANDLE_VALUE) return(COMMUNICATIONS_ERROR);

	fSuccess = GetCommState(hCom, &dcb);
	if (!fSuccess) return(COMMUNICATIONS_ERROR);

	if (nOnOff)	dcb.fDtrControl=DTR_CONTROL_ENABLE;
	else dcb.fDtrControl=DTR_CONTROL_DISABLE;
	
	fSuccess = SetCommState(hCom, &dcb);
	
	if (!fSuccess) return(COMMUNICATIONS_ERROR);
	else return(STATUS_OK);

}

/*********************************************************************                      
* Synopsis:       NoMangle long DLL_IMPORT_EXPORT csp2WakeUp(void)
*
* Description:    If DTR is set, clears it momentarily, then sets it again. 
*                 Otherwise just set it.     
*
* Parameters:       
*
* Return Value:     
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2WakeUp(void)
{
int retval,k,i;
struct _timeb now;

    // See if DTR is active
	retval=GetDTR();
	if (retval<0) return(COMMUNICATIONS_ERROR);
	if (retval>0) 
	{
	  // DTR is currently set, so clear it
	  retval=SetDTR(HS_CLEAR);
	  if (retval<0) return(COMMUNICATIONS_ERROR);

      // delay a little before setting DTR
      Sleep ( 175 );

	}

	_ftime(&now);
	if ((now.time-PwrDwnTime.time)<2){
		k=now.millitm;
		if (now.time!=PwrDwnTime.time) k+=1000;
		i=CS1504_PWR_UP_TIME-(k-PwrDwnTime.millitm);
//++++
#ifdef _DEBUG
		if (bCspDebugMode)
		{
			nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
				"\r\n\r\nWakeUp Timing Info: %d.%d %d.%d %d.\r\n", 
				now.time,now.millitm,PwrDwnTime.time,PwrDwnTime.millitm,i);
			WriteFile(  hCspDebug,          // handle to debug text file
				szCspDebug,         // buffer of data to write to file
				nCspDebugLength,    // number of bytes to write
				&nCspDebugLength,   // number of bytes written
				0);                 // no overlapped structure present
		}
#endif

//++++
		if (i>0) Sleep(i);
	}
	retval=SetDTR(HS_SET);
	if (retval<0) return(COMMUNICATIONS_ERROR);
	else return(STATUS_OK);
}

/*********************************************************************                      
* Synopsis:       NoMangle long DLL_IMPORT_EXPORT csp2GetCTS(void) 
*
* Description:      
*
* Parameters:       
*
* Return Value:   <0 : Error.  0 : CTS is clear.  >0 : CTS is set.
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetCTS(void)
{

	return(GetCTS());

}

/*********************************************************************                      
* Synopsis:       NoMangle long DLL_IMPORT_EXPORT csp2SetDTR(long nOnOff)
*
* Description:      
*
* Parameters:       
*
* Return Value:     
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2SetDTR(long nOnOff)
{
int retval;

	if (nOnOff) retval=SetDTR(HS_SET);
	else retval=SetDTR(HS_CLEAR);
	return((long)retval);

}

/*********************************************************************                      
* Synopsis:       
*
* Description:    This routine assumes that the DTR line is CLEAR. If it is not, 
*                 the device may have already timed out and gone to sleep.
*
* Parameters:       
*
* Return Value:  0 for no data available, >0 for data available.   
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2DataAvailable(void)
{
int retval;

    // make sure we have a valid port selected...
    if (nCspActivePort < COM1)
        return( COMMUNICATIONS_ERROR );

    // See if DTR is active
	retval=GetDTR();
	if (retval<0) return(COMMUNICATIONS_ERROR);
	if (retval==0)
	{
	    retval=SetDTR(HS_SET);
	    if (retval<0) return(COMMUNICATIONS_ERROR);
        // wait a little before checking line.
        Sleep ( 175 ); 
	}

	// Get the data ready line (CTS) status
	// retval=SioCTS(nCspActivePort); 
	retval=GetCTS(); 
	
	return(retval);

}

/*********************************************************************                      
* Synopsis:        long DLL_IMPORT_EXPORT csp2Init(long nComPort)
*
* Description:    This function tries to open the communications port
*                 specified by nComPort. f it is successful, the
*                 port is configured for CSP device compatibility.
*
*                 This provides a single call to initialize the CSP
*                 commnications and all of the software structures 
*                 for the CSP interface. The intialization consists of:
*                    1. Initialize all memory structures, queues, and pointers
*                    2. Establish serial communications with the specified port  
*
* Parameters:     nComPort specifies a communications port such as
*                 COM1, Com2, etc.  
*
* Return Value:   STATUS_OK if the specified nComPort is now active
*                 BAD_PARAM if nComPort is invalid
*                 COMMUNICATIONS_ERROR if nCommPort could not be activated  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2Init(long nComPort)
{
const char Ports[16][6]={
		"COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8",
			"COM9","COM10","COM11","COM12","COM13","COM14","COM15","COM16"};

COMMTIMEOUTS TimeOuts;

    // is the user requesting a valid port?
    if ((nComPort >= COM1)&&(nComPort<=COM16))
        {
        // close any previously opened ports...
        if (nCspActivePort >= COM1) csp2Restore();//CloseHandle(hCom);// should restore here

		memset(&PwrDwnTime,0,sizeof(PwrDwnTime));
		hCom = CreateFile( Ports[nComPort],
			GENERIC_READ | GENERIC_WRITE,
			0,    // comm devices must be opened w/exclusive-access 
			NULL, // no security attributes 
			OPEN_EXISTING, // comm devices must use OPEN_EXISTING 
			0,    // not overlapped I/O 
			NULL  // hTemplate must be NULL for comm devices 
			);
		
		if (hCom == INVALID_HANDLE_VALUE) return(COMMUNICATIONS_ERROR);

		// Set buffer sizes

		fSuccess = SetupComm(hCom,RX_QUE_SIZE,TX_QUE_SIZE);
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}
		
		// Omit the call to SetupComm to use the default queue sizes.
		// Get the current configuration.
		
// Save original DCB
/*
		fSuccess = GetCommState(hCom, &OriginalDcb);
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}
*/
// ok, now set up for our purposes
		fSuccess = GetCommState(hCom, &dcb);
		
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}
		
		memcpy(&OriginalDcb,&dcb,sizeof(dcb)); // save original dcb
		// Fill in the DCB: baud=9600, 8 data bits, no parity, 1 stop bit. 
		
		dcb.BaudRate = 9600;
		dcb.ByteSize = 8;
		dcb.Parity = ODDPARITY;
		dcb.StopBits = ONESTOPBIT;
		dcb.fDtrControl = DTR_CONTROL_ENABLE;
        dcb.fOutxCtsFlow = FALSE;
        dcb.fDsrSensitivity = FALSE;
		
		fSuccess = SetCommState(hCom, &dcb);
		
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}

/* !!!  A note on Comm TimeOuts  !!!
    A value of MAXDWORD for ReadIntervalTimeout, combined with zero
	values for both the ReadTotalTimeoutConstant and
	ReadTotalTimeoutMultiplier members, specifies that the read
	operation is to return immediately with the characters that have
	already been received, even if no characters have been received. 
*/
		fSuccess = GetCommTimeouts(hCom,&TimeOuts);
		
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}

		TimeOuts.ReadIntervalTimeout=MAXDWORD;
		TimeOuts.ReadTotalTimeoutConstant=NULL;
		TimeOuts.ReadTotalTimeoutMultiplier=NULL;

		fSuccess = SetCommTimeouts(hCom,&TimeOuts);
		
		if (!fSuccess) 
		{
			// Handle the error.
			CloseHandle(hCom);
			hCom = INVALID_HANDLE_VALUE;
			return(COMMUNICATIONS_ERROR);
		}

		nCspActivePort = nComPort;
	
        return (STATUS_OK);
        }
        
    return (BAD_PARAM);
}

/*********************************************************************                      
* Synopsis:        long DLL_IMPORT_EXPORT csp2Restore()
*
* Description:    This function attempts to close open connections
*                 to the serial port specified by nCspActivePort.  
*
* Parameters:     None  
*
* Return Value:   STATUS_OK if the port was closed  
*                 COMMUNICATIONS_ERROR if the connection was never
*                 made or cannot be closed
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2Restore()
{ 
    long nRetStatus = COMMUNICATIONS_ERROR;
    fSuccess=NULL;
    // are we attempting to close a valid port?
    if (nCspActivePort >= COM1)
	{
#ifdef _DEBUG
		if (bCspDebugMode)
		{
			nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
				"\r\nClosing ComPort %d.\r\n", 
				nCspActivePort+1);
			WriteFile(  hCspDebug,  // handle to debug text file
				szCspDebug,         // buffer of data to write to file
				nCspDebugLength,    // number of bytes to write
				&nCspDebugLength,   // number of bytes written
				0);                 // no overlapped structure present
		}
#endif
        // close any previously opened ports...
		if (hCom!=INVALID_HANDLE_VALUE)
		{
			fSuccess = SetCommState(hCom, &OriginalDcb);
		}
		fSuccess = CloseHandle(hCom);
		hCom = INVALID_HANDLE_VALUE;
	} 
	
    // initialize the dll interface...
    csp2InitParms();
	
    // return status...
    if (!fSuccess )
        return (COMMUNICATIONS_ERROR);
    else
        return (STATUS_OK);
}

/*********************************************************************                      
* Synopsis:        long DLL_IMPORT_EXPORT csp2ReadData()
*
* Description:    This function reads the following information stored
*                  in the CSP device:
*                      All Barcodes
*                      Device ID
*                      Signature
*
*                  The information is stored within the DLL. In order 
*                  to retreive the information, the user must call the 
*                  corresponding functions listed below:
*                      csp2GetPacket()
*                      csp2GetDeviceId()
*                      csp2GetSignature()
*
* Parameters:     None  
*
* Return Value:   Number of stored barcode strings if no error, otherwise:
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2ReadData()
{
    long nRetStatus;
	char ParamValue[2];

	// wake up the device
	nRetStatus = csp2WakeUp();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );

   	// Interrogate the device
	nRetStatus = csp2Interrogate();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );

	// Get Ascii Mode setting
	nRetStatus = GetParam(Param_ASCII_Mode,ParamValue,1);
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	nCspASCII = (long) ParamValue[0];

	// Get TimeStamp Setting
	nRetStatus = GetParam(Param_Store_RTC,ParamValue,1);
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	nCspRTC = (long) ParamValue[0];

#ifdef _DEBUG
		if (bCspDebugMode)
		{
			nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
				"\r\nAscii Mode: %d, RTC Mode %d.\r\n", 
				nCspASCII,nCspRTC);
			WriteFile(  hCspDebug,  // handle to debug text file
				szCspDebug,         // buffer of data to write to file
				nCspDebugLength,    // number of bytes to write
				&nCspDebugLength,   // number of bytes written
				0);                 // no overlapped structure present
		}
#endif
	
	
	// read the data from the CSP device
    nRetStatus = csp2ReadRawData(NULL, DETERMINE_SIZE);

    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );

	// shut down the device

	nRetStatus = csp2PowerDown();

    return( nCspStoredBarcodes );
}


/*********************************************************************                      
* Synopsis:        long DLL_IMPORT_EXPORT csp2ClearData()
*
* Description:     Clear all of the bar codes stored in the CSP device.  
*
* Parameters:       
*
* Return Value:    STATUS_OK
*                  COMMUNICATIONS_ERROR
*                  INVALID_COMMAND_NUMBER
*                  COMMAND_LRC_ERROR
*                  RECEIVED_CHARACTER_ERROR
*                  GENERAL_ERROR 
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aClearBarCodesCmd[] =
    {
    CLEAR_BAR_CODES,         // Clear Bar Code Command
    STX,                     // opcode
    0x00,                    // NULL terminate the message
    0x9f,                    // CRC HH
	0x2e					 // CRC LL
    };

NoMangle long DLL_IMPORT_EXPORT csp2ClearData()
{
    long i;
    long nRetStatus;
    
	// wake up the device
	nRetStatus = csp2WakeUp();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
   	// Interrogate the device
	nRetStatus = csp2Interrogate();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
    
    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nClear Barcodes: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand ((char *)aClearBarCodesCmd, sizeof(aClearBarCodesCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // get the NULL character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    NULL:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );

    nRetStatus=csp2PowerDown();
    
	return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2SetDefaults(void)
*
* Description:    Sets defaults on the device  
*
* Parameters:     None  
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aSetDefaultsCmd[] =
   {
   SET_DEFAULTS,            // Set Defaults Command
   STX,                     // opcode
   0x01,
   0x01,
   0x00,                    // NULL terminate the message
   0xd7,                    // CRC HH
   0x7b						// CRC LL
   };

NoMangle long DLL_IMPORT_EXPORT csp2SetDefaults(void)
{
    long i,k;
    long nRetStatus;
	unsigned char j;

	// wake up the device
	nRetStatus = csp2WakeUp();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
   	// Interrogate the device
	nRetStatus = csp2Interrogate();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
	// send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nSet Defaults: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand ((char *)aSetDefaultsCmd, sizeof(aSetDefaultsCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

	// get counted string response
    while (( aByteBuffer[i++] = j = (char) csp2Getc()) > 0x00 )
        {
        // read the value into aByteBuffer[] upto j characters...
        for (k = 0; k < (long) (j); k++)
            {
            aByteBuffer[i++] = (char) csp2Getc();
            }

        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}


/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2SetTime(unsigned char aTimeBuf[]) 
*
* Description:    Sets the current time on the device  
*
* Parameters:     aTimeBuf[] is the array where the 6 bytes of time data are taken.  
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR  
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aSetTimeCmd[] =
   {
   SET_TIME,                // Set Time Command
   STX,                     // opcode
   0x00,                    // NULL terminate the message
   0x5d,                    // CRC HH
   0xaf						// CRC LL
   };

NoMangle long DLL_IMPORT_EXPORT csp2SetTime(unsigned char aTimeBuf[])
{
    long nRetStatus;
    long i;
    long k;
    char j;

	// wake up the device
	nRetStatus = csp2WakeUp();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
    // see if the CSP device is connected...
    if ((nRetStatus = csp2Interrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // build the download packet...
	i=0;
    aByteBuffer[i++] = SET_TIME;
    aByteBuffer[i++] = STX;

    // set the length of the counted string...
    aByteBuffer[i++] = 0x06;

     // copy the string into the buffer...
    for (k = 0; k < 6; k++)
        aByteBuffer[i++] = aTimeBuf[k];

    // null terminate all strings, assumed
    aByteBuffer[i++] = 0x00;    

    // perform CRC Check on the string...

	CRCtest.w=ComputeCRC16((unsigned char *)aByteBuffer,i);
    aByteBuffer[i++] = CRCtest.b.hi;
    aByteBuffer[i++] = CRCtest.b.lo;


    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nSet Time: %s [Time: %s]\r\n", 
                                        lpszCspDebug, 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand( aByteBuffer, i );

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    while (( aByteBuffer[i++] = j = (char) csp2Getc()) > 0x00 )
        {
        // get the time character...
        aByteBuffer[i++] = (char) csp2Getc();

        // read the time value into aByteBuffer[] upto j-1 characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            aByteBuffer[i++] = (char) csp2Getc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

// verify CRC 
	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetTime(unsigned char aTimeBuf[])
*
* Description:    Get the current time from the device  
*
* Parameters:     aTimeBuf[] is the array where data will be copied after it
*                 is read out of the Device. THIS ARRAY IS ASSUMED TO BE AT 
*                 LEAST 6 BYTES IN LENGTH.
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aGetTimeCmd[] =
   {
   GET_TIME,                // Get Time Command
   STX,                     // opcode
   0x00,                    // NULL terminate the message
   0x5d,                    // CRC HH
   0xaf						// CRC LL
   };

NoMangle long DLL_IMPORT_EXPORT csp2GetTime(unsigned char aTimeBuf[])
{
    long i,j;
    long nRetStatus;

	// wake up the device
	nRetStatus = csp2WakeUp();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
   	// Interrogate the device
	nRetStatus = csp2Interrogate();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
	    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nGetTime: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand ((char *)aGetTimeCmd, sizeof(aGetTimeCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // get the count (6) character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    0x06:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

	// get the time data
    for ( j = 0; j < 6 ; j++ )
        aByteBuffer[i++] = aTimeBuf[j] = (char) csp2Getc();


    // get the NULL character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    NULL:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );

    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2PowerDown()
*
* Description:    Direct the Device to end communication and enter
*                 STOP mode until the next trigger action. This function
*                 returns on or before the Device issues an audible signal
*                 confirming the power down command.
*
* Parameters:       
*
* Return Value:  STATUS_OK
*                COMMUNICATIONS_ERROR
*                INVALID_COMMAND_NUMBER
*                COMMAND_LRC_ERROR
*                RECEIVED_CHARACTER_ERROR
*                GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aPowerDownCmd[] =
   {
   POWER_DOWN,              // Power down Command
   STX,                     // opcode
   0x00,                    // NULL terminate the message
   0x5e,                    // CRC HH
   0x9f						// CRC LL
   };

NoMangle long DLL_IMPORT_EXPORT csp2PowerDown()
{
    long i;
    long nRetStatus;

   	// Interrogate the device
	nRetStatus = csp2Interrogate();
    // if response not available, reply with error
    if ( nRetStatus < 0 )
        return ( nRetStatus );
	
    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nPower Down: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand ((char *)aPowerDownCmd, sizeof(aPowerDownCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // get the NULL character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    NULL:             ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );

// It worked, note the time
	_ftime(&PwrDwnTime);

    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetPacket(char szBarData[], 
*                                                      long nBarcodeNumber, 
*                                                      long nMaxLength)
*
* Description:      
*
* Parameters:     szBarData[] is a character array the user has allocated 
*                 to hold the barcode data. The string will be null terminated.
*                 nBarcodeNumber indicates which barcode stored in
*                 the szCspBcString[] will be returned.
*                 nMaxLength is the length of the allocated space including the
*                 null terminator. If nMaxLength is set to DETERMINE_SIZE, the
*                 function will return the length of the barcode without
*                 copying any data. 
*
* Return Value:   > 0  length of the barcode including the NULL terminator
*                 BAD_PARAM if the requested barcode does not exist
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetPacket(char szBarData[], long nBarcodeNumber, long nMaxLength)
{
    long i;
    long nBarFound;
    long nBarLength;
    
    // make sure the requested barcode is valid...
    if (nBarcodeNumber < 0)
        return( BAD_PARAM );

    if (nBarcodeNumber >= nCspStoredBarcodes)
        return( BAD_PARAM );

    // OK, go find the barcode...
    i = 0;
    nBarFound = 0;
    while ((nBarFound < nBarcodeNumber) && (i<MAXSIZE))
        {
		// Add next string length (+1) to current index to index next counted string
		i+= szCspBarData[i]+1; 
        
        // bump count to the next barcode...
        nBarFound++;
        }   

    // OK, we've got the barcode, so process it...
    nBarLength = szCspBarData[i]+1;

    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(nBarLength,nMaxLength);

        // copy the barcode...
        memcpy(szBarData, &szCspBarData[i], nMaxLength);
        }

    return(nBarLength);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetDeviceId(char szDeviceId[9], 
*                                                        long nMaxLength)
*
* Description:    This function retrieves the Device ID array from the CSP
*                 device data stored in the DLL. This is a binary array.
*
* Parameters:     szDeviceId[] holds the returned data.
*                 nMaxLength is the length of the allocated space. 
*
* Return Value:   The length of the Device ID array. This is not a string! 
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetDeviceId(char szDeviceId[8], long nMaxLength)
{
    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(szCspDeviceId),nMaxLength);

        // copy the Device ID...
        memcpy(szDeviceId, szCspDeviceId, nMaxLength);
        return( nMaxLength );
        }

    return( sizeof(szCspDeviceId) );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetRTCMode()
*
* Description:    Returns an integer corresponding to the Store_RTC
*                 parameter from the CSP device.
*
* Parameters:     None  
*
* Return Value:   >= Current RTC storage information
*                 < 0, RTC storage mode not available
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetRTCMode()
{
    return (nCspRTC);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetASCIIMode()
*
* Description:    Returns an integer corresponding to the ASCII_MODE
*                 parameter from the CSP device.
*
* Parameters:     None  
*
* Return Value:   >= Current ASCII_MODE information
*                 < 0, ASCII_MODE not available
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetASCIIMode()
{
    return (nCspASCII);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetProtocol()
*
* Description:    Returns an integer corresponding to the Protocol
*                 byte from the CSP device.
*
* Parameters:     None  
*
* Return Value:   >= Current Protocol version information
*                 < 0, Protocol version not available
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetProtocol()
{
    return (nCspProtocolVersion);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetSystemStatus()
*
* Description:    Returns an integer corresponding to the System 
*                 Status byte from the CSP device.
*
* Parameters:     None 
*
* Return Value:   >= Current System Status information
*                 < 0, System Status not available
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetSystemStatus()
{
    return (nCspSystemStatus);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetSwVersion(char szSwVersion[9], 
*                                                         long nMaxLength) 
*
* Description:    This function retrieves the Software Version string from the CSP
*                 device data stored in the DLL. The string is null terminated.
*
* Parameters:     szSwVersion[] holds the returned data.
*                 nMaxLength is the length of the allocated space including
*                 the null terminator.
*
* Return Value:   Length of Software Version string including null terminator.
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetSwVersion(char szSwVersion[9], long nMaxLength)
{
    // did the user only request the string's length?
    if (nMaxLength > DETERMINE_SIZE)
        {
        // get the maximum number of characters to copy...
        nMaxLength = min(sizeof(szCspSwVersion),nMaxLength);

        // copy the Software Version...
        memcpy(szSwVersion, szCspSwVersion, nMaxLength);
        
        // and null terminate the string...
        szSwVersion[nMaxLength-1] = 0;
        }

    return( sizeof(szCspSwVersion) );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2SetRetryCount(long nRetryCount)
*
* Description:    This function sets the communications retry count. The 
*                 default setting is 5. In the event of a communications
*                 failure, the DLL will attempt the device interrogation
*                 nRetryCount more times before returning a communications
*                 error. This allows the user time to activate the CSP
*                 device and connect it to the system in a reasonable time
*                 period.
*
* Parameters:     nRetryCount specifies the number of interrogation retries.
*                 The valid range for nRetryCount is [0..9]
*
* Return Value:   STATUS_OK
*                 BAD_PARAM if nRetryCount not in range of valid values.
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2SetRetryCount(long nRetryCount)
{
    // make sure the retry count is within the valid range...
    if ((nRetryCount < 0) || (nRetryCount > 9))
        return( BAD_PARAM );
        
    // change the retry count as indicated
    nCspRetryCount = nRetryCount;

    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetRetryCount()
*
* Description:    This function returns the current value of nRetryCount.
*
* Parameters:     None  
*
* Return Value:   The value of nRetryCount
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetRetryCount()
{
    return (nCspRetryCount);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetDllVersion(char szDllVersion[], 
*                                                          long nMaxLength)
*
* Description:    If this function is part of a DLL, it will return the 
*                 FileVersion information about the DLL.
*
*                 If this function is statically linked with an application,
*                 then the FileVersion information for the application is
*                 returned.
*
*                 The correct operation of this function is tied to the 
*                 Resource Information provided by VERSIONINFO in the 
*                 FileVersion string. Modify the appropriate resource file
*                 to change this setting.
*
* Parameters:     szDllVersion[] holds the returned string.
*                 nMaxLength is the length of the allocated space including
*                 the null terminator.
*
* Return Value:   STATUS_OK           FileVersion info was read and returned
*                 FILE_NOT_FOUND      FileVersion info could not be obtained
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetDllVersion(char szDllVersion[], long nMaxLength)
{
    char    fileName[_MAX_PATH];    // name of this executable code
    BYTE    block[2048];            // information block about this file
    DWORD   FAR *translation;       // pointer to the string translation(s)
    DWORD   FAR *buffer;            // a pointer into the FileVersion string
    DWORD   bytes;                  // number of bytes read
    DWORD   handle;                 // a handle to the file
    char    name[64];               // a name of the string to retreive

    //  Note!
    //
    //      If this DLL is dynamically linked against the MFC
    //      DLLs, any functions exported from this DLL which
    //      call into MFC must have the AFX_MANAGE_STATE macro
    //      added at the very beginning of the function.
    //
    //      For example:
    //
    //      extern "C" BOOL PASCAL EXPORT ExportedFunction()
    //      {
    //          AFX_MANAGE_STATE(AfxGetStaticModuleState());
    //          // normal function body here
    //      }
    //
    //      It is very important that this macro appear in each
    //      function, prior to any calls into MFC.  This means that
    //      it must appear as the first statement within the 
    //      function, even before any object variable declarations
    //      as their constructors may generate calls into the MFC
    //      DLL.
    //
    //      Please see MFC Technical Notes 33 and 58 for additional
    //      details.
    //
    //AFX_MANAGE_STATE(AfxGetStaticModuleState());

    // Get the filename of THIS DLL!
    GetModuleFileName(hModuleThis, fileName, _MAX_PATH);

    // Get the size of the information block for this dll...
    bytes = GetFileVersionInfoSize(fileName, &handle);

    // is there any information?
    if (bytes)
        {
        // make sure we don't read too much data...
        if (bytes > 2048)
            bytes = 2048;

        // Get the actual block for the version information into memory...
        if ( GetFileVersionInfo(fileName, handle, bytes, block) )
            {
            // Get the File Version for THIS DLL by name...
            if ( VerQueryValue(block, "\\VarFileInfo\\Translation", 
                              (VOID FAR * FAR *) &translation, 
                              (UINT FAR *) &bytes) )
                {
                // Now that we have the "block" to translate, ask for the FileVersion...
                // Format a request for the FileVersion string...
                wsprintf(name, "\\StringFileInfo\\%04x%04x\\FileVersion",
                         LOWORD(*translation), HIWORD(*translation));

                // Go get the FileVersion string from the block...
                if ( VerQueryValue(block, name, (VOID FAR * FAR *) &buffer, (UINT FAR *) &bytes) )
                    {
                    // return up to nMaxLength characters of the version information string...
                    #ifdef _DEBUG
                        if (nMaxLength >= (long) (bytes+6))
                            {
                            memcpy( (LPSTR) szDllVersion, (LPSTR) "Debug ", nMaxLength);
                            strncat( (LPSTR) szDllVersion, (LPSTR) buffer, nMaxLength-6);
                            }
                        else
                            return(bytes+6);    // String would not fit!
                    #else
                        if (nMaxLength >= (long) bytes)
                            memcpy( (LPSTR) szDllVersion, (LPSTR) buffer, nMaxLength);
                        else
                            return(bytes);      // String would not fit!
                    #endif

                    // Say that everything was OK with the version information...
                    return(STATUS_OK);
                    }
                }

            }

        }

    // We had a problem somewhere, send back a COMMUNICATIONS_ERROR for status...
    return(FILE_NOT_FOUND);
}

/*********************************************************************                      
* Synopsis:        long DLL_IMPORT_EXPORT csp2SetDebugMode(long nOnOff)
*
* Description:     This function turns the debug mode of the DLL on or off.
*                  
*                  When the debug mode is on, all commands and reponses are written 
*                  to a debug file named "debug.txt". Each entry in the file
*                  will have a time stamp preceeding it.
*
*                  Note: this function function only writes data to the 
*                  output file in the debug version of the DLL.
*
* Parameters:      nOnOff is one of the following values:
*                  PARAM_ON
*                  PARAM_OFF
*
* Return Value:    STATUS_OK
*                  FILE_NOT_FOUND
*                  other: see WinError.h
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2SetDebugMode(long nOnOff)
{
    #ifdef _DEBUG
    // are we turning off the debug file?
    if (nOnOff == PARAM_OFF)
        {
        // did we open a file previously?
        if (bCspDebugMode)
            {
            // flag debug mode as terminated...
            bCspDebugMode = FALSE;

            // try to close the file...
            if ( !CloseHandle( hCspDebug ) )
                return( (long) GetLastError() );
            }
        else
            return( FILE_NOT_FOUND );
        }
    else
        {
        // create the debug file...
        hCspDebug = CreateFile("Debug.txt",                 // debug file name
                                GENERIC_WRITE,              // we are going to write this file
                                0,                          // file sharing disabled
                                0,                          // use default security descriptor
                                CREATE_ALWAYS,              // re-write previously existing file
                                FILE_ATTRIBUTE_NORMAL,      // default file attributes
                                0);                         // no template file provided

        // did we create the file successfully?
        if ( !hCspDebug )
            {
            // return the reason for the file creation error...
            return( (long) GetLastError() );            
            }

        // flag debug mode as active...
        bCspDebugMode = TRUE;

        nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                    "Consumer Scanning Products Debug File\r\n"
									"Initial Comm timeout = %d ms\r\n",
									 (MAXTIME*50));
        WriteFile(  hCspDebug,          // handle to debug text file
                    szCspDebug,         // buffer of data to write to file
                    nCspDebugLength,    // number of bytes to write
                    &nCspDebugLength,   // number of bytes written
                    0);                 // no overlapped structure present
        }
    #endif

    // everything is OK
    return (STATUS_OK);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2GetCommInfo(long nComPort)
*
* Description:    This function stub is retained for compatability with CSP
*                 programs only.
*
* Parameters:     nCommPort is in the range [COM1..COM16]
*
* Return Value:     
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2GetCommInfo(long nComPort)
{
    return (0);
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT csp2ReadRawData(char aBuffer[], 
*                                                        long nMaxLength )
*
* Description:    This function performs an "Upload" command on a CSP device. 
*                 The raw data read from the CSP device is placed into 
*                 aBuffer[] for up to nMaxLength bytes.  If the amount of 
*                 data that is read from the CSP device exceeds nMaxLength, 
*                 the buffer will be truncated at nMaxLength bytes. 
*
*                 This function will retry the interrogation upto
*                 nCspRetryCount times before aborting the operation.
*
*                 Protocol Change effective 4/1/98: Signature field may
*                 or may not be present: it can be disabled. This code now
*                 takes that in to account.
*
*                 This function asks the Device if the signature is enabled
*                 before it starts an upload. So long as the comunications
*                 works, it performs the upload.
*
*                 If the signature is enabled, and if the last counted 
*                 string is 8 characters long, it is assumed to be the 
*                 signature string.  
*
*                 If it is not eight characters long, or the signature 
*                 is disabled, then the Signature field is returned as 
*                 'Disabled'.
*
* Parameters:     aBuffer[] array where data will be copied after it
*                 is read out of the Device.
*                 nMaxLength is the length of the allocated array.
*                 If nMaxLength is set to DETERMINE_SIZE, the function will 
*                 return the length of the barcode without copying any data.
*
* Return Value:   Number of bytes read if no error, otherwise:
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aUploadCmd[] =
    {
    UPLOAD,                  // Upload Command
    STX,                     // opcode
    0x00,                    // NULL terminate the message
    0x9e,                    // CRC HH
	0x3e					 // CRC LL
    };

NoMangle long DLL_IMPORT_EXPORT csp2ReadRawData(char aBuffer[], long nMaxLength )
{
    long i, k, n;
    long nRetStatus;
    char j;

    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nRead Data: [Time: %s]\r\n", 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand ((char *)aUploadCmd, sizeof(aUploadCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // get the Device id characters...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Device ID:        ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    for ( j = 0; j < 8 ; j++ )
        aByteBuffer[i++] = szCspDeviceId[j] = (char) csp2Getc();

    // read in the counted strings until a NULL occurs...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nCspStoredBarcodes = k = n = 0;
    while (( aByteBuffer[i++] = szCspBarData[k++] = j = (char) csp2Getc()) > 0x00 )
        {
        // get the bar code string characters...
         for ( n = 0; n < (long) j; n++ )
            aByteBuffer[i++] = szCspBarData[k++] = (char) csp2Getc();

        nCspStoredBarcodes++;

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

	aByteBuffer[i++]=(char) csp2Getc();
	aByteBuffer[i] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, (i-1)) != TRUE)
		return( COMMAND_LRC_ERROR );

    // should we copy the data to the user's array?
    if (aBuffer)
        {
        if (nMaxLength)
            memcpy(aBuffer, aByteBuffer, nMaxLength);
        }

    return(i + 1);	  // Return byte count
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT  csp2SetMultiParam( char *szString, 
*                                                            long nMaxLength )
*
* Description:    The csp2SetMultiParam function permits the user to write 
*                 multiple parameters at one time rather than individual
*                 downloads.
*
* Parameters:     *string points to a buffer of multiple parameter strings.
*					   Each parameter and value must be present in the counted 
*                 string format within the buffer. The trailing CRC will
*					   be added by the function call.
*                 length specifies how many characters in string[] make
*                 up the new parameter setting.
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2SetMultiParam( char szString[], long nMaxLength )
{
    long nRetStatus;
    long i = 0;
    long k;
    char j;

	if (nMaxLength>MAX_XMIT_SIZE) 
		return(GENERAL_ERROR);
    // see if the CSP device is connected...
    if ((nRetStatus = csp2Interrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // start the download packet...
    aByteBuffer[i++] = DOWNLOAD_PARAMETERS;
    aByteBuffer[i++] = STX;

    // copy the string into the buffer...
    for (k = 0; k < nMaxLength; k++)
        aByteBuffer[i++] = szString[k];

    // null terminate all strings, assumed
    aByteBuffer[i++] = 0x00;    

    // perform CRC Check on the string...

	CRCtest.w=ComputeCRC16((unsigned char *)aByteBuffer,i);
    aByteBuffer[i++] = CRCtest.b.hi;
    aByteBuffer[i++] = CRCtest.b.lo;


    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nSet Parameter: %s [Time: %s]\r\n", 
                                        lpszCspDebug, 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand( aByteBuffer, i );

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    while (( aByteBuffer[i++] = j = (char) csp2Getc()) > 0x00 )
        {
        // get the parameter character...
        aByteBuffer[i++] = (char) csp2Getc();

        // read the parameter value into aByteBuffer[] upto j-1 characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            aByteBuffer[i++] = (char) csp2Getc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

// verify CRC 
	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );


    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT  csp2SetParam( long nParam, 
*                                                       char *szString, 
*                                                       long nMaxLength )
*
* Description:    The csp2SetParam function permits the user to write 
*                 individual Device parameters one at a time rather than 
*                 batching all of the parameters for one download.
*
* Parameters:     param_number is the CSP device parameter to change
*                *string points to the new parameter setting
*                length specifies how many characters in string[] make
*                up the new parameter setting.
*
* Return Value:  STATUS_OK
*                COMMUNICATIONS_ERROR
*                INVALID_COMMAND_NUMBER
*                COMMAND_LRC_ERROR
*                RECEIVED_CHARACTER_ERROR
*                GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
NoMangle long DLL_IMPORT_EXPORT csp2SetParam( long nParam, char szString[], long nMaxLength )
{
    long nRetStatus;
    long i = 0;
    long k;
    char j;

    // see if the CSP device is connected...
    if ((nRetStatus = csp2Interrogate()) != STATUS_OK)
        return( nRetStatus );
        
    // start the download packet...
    aByteBuffer[i++] = DOWNLOAD_PARAMETERS;
    aByteBuffer[i++] = STX;

    // set the length of the counted string...
    aByteBuffer[i++] = nMaxLength + 1;

    // specify the parameter number...
    aByteBuffer[i++] = (char) nParam;

    // copy the string into the buffer...
    for (k = 0; k < nMaxLength; k++)
        aByteBuffer[i++] = szString[k];

    // null terminate all strings, assumed
    aByteBuffer[i++] = 0x00;    

    // perform CRC Check on the string...

	CRCtest.w=ComputeCRC16((unsigned char *)aByteBuffer,i);
    aByteBuffer[i++] = CRCtest.b.hi;
    aByteBuffer[i++] = CRCtest.b.lo;


    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nSet Parameter: %s [Time: %s]\r\n", 
                                        lpszCspDebug, 
                                        szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand( aByteBuffer, i );

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;

    // get the STX character...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    aByteBuffer[i++] = (char) csp2Getc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    while (( aByteBuffer[i++] = j = (char) csp2Getc()) > 0x00 )
        {
        // get the parameter character...
        aByteBuffer[i++] = (char) csp2Getc();

        // read the parameter value into aByteBuffer[] upto j-1 characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            aByteBuffer[i++] = (char) csp2Getc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

// verify CRC 
	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
		return( COMMAND_LRC_ERROR );


    return( STATUS_OK );
}
   
/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT  csp2GetParam( long nParam, 
*                                                       char *szString, 
*                                                       long nMaxLength )
*
* Description:    The csp2GetParam function permits the user to read
*                 individual Device parameters one at a time.
*
* Parameters:     nParam is the CSP device parameter to retrieve
*                 *szString stores the retrieved parameter value
*                 nMaxLength specifies the maximum number of characters that
*                 can be stored in *string.
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/

NoMangle long DLL_IMPORT_EXPORT csp2GetParam( long nParam, char szString[], long nMaxLength )
{
    long nRetStatus;

    // see if the CSP device is connected...
    if ((nRetStatus = csp2Interrogate()) != STATUS_OK)
        return( nRetStatus );
	return(GetParam(nParam, szString, nMaxLength));
}
		
/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT  GetParam( long nParam, 
*                                                   char *szString, 
*                                                   long nMaxLength )
*
* Description:    The GetParam is the underlying functionality for the 
*                 csp2GetParam function. It permits the user to read
*                 individual Device parameters one at a time.
*
* Parameters:     nParam is the CSP device parameter to retrieve
*                 *szString stores the retrieved parameter value
*                 nMaxLength specifies the maximum number of characters that
*                 can be stored in *string.
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
char aGetParametersCmd[] =
    {
    UPLOAD_PARAMETERS,       
    STX,                     

    0x01,                    // Read one parameter
    0x00,                    // TBD parameter

    0x00,                    // NULL terminate the message
    0x00,                    // CRC HH
	0x00					 // CRC LL
    };

#define GPC_PARM_START  ((long) 3)
#define GPC_CRC			(sizeof(aGetParametersCmd) - 2)
#define GPC_SIZE        (sizeof(aGetParametersCmd) - 1)

static long GetParam( long nParam, char szString[], long nMaxLength )
{
    long i, k;
    long nRetStatus;
    char j;

    // fill in the command string...
    aGetParametersCmd[GPC_PARM_START] = (char) nParam;
	CRCtest.w=ComputeCRC16((unsigned char *)aGetParametersCmd, GPC_CRC);
	aGetParametersCmd[GPC_CRC]=CRCtest.b.hi;
	aGetParametersCmd[GPC_CRC+1]=CRCtest.b.lo;
	
    // send the command to the CSP device...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            _strtime( szDebugTime );        // Get time stamp string
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n\r\nGet Parameter %d: [Time: %s]\r\n",
										 nParam,
                                         szDebugTime);
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    nRetStatus = csp2SendCommand (aGetParametersCmd, sizeof(aGetParametersCmd));

    if ( nRetStatus != STATUS_OK )
        return ( nRetStatus );

    // Response looks good, get the entire message...
    i = 1;
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    STX:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
    // get the STX character...
	aByteBuffer[i++] = (char) csp2Getc();

    // read in the counted string until a NULL occurs...
    // first byte "j" is the length of the counted string...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    Counted String:   ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif

    while (( aByteBuffer[i++] = j = (char) csp2Getc()) > 0x00 )
        {
        // get the parameter character...
        aByteBuffer[i++] = (char) csp2Getc();

        // read the parameter value into szString[] upto nMaxLength characters...
        for (k = 0; k < (long) (j-1); k++)
            {
            if (k < nMaxLength)
                aByteBuffer[i++] = szString[k] = (char) csp2Getc();
            // parameter exceeded szString[], don't copy over buffer...
            else
                aByteBuffer[i++] = (char) csp2Getc();
            }

        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n                      ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        }

    // verify the CRC...
    #ifdef _DEBUG
        if (bCspDebugMode)
            {
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                        "\r\n    CRC:              ");
            WriteFile(  hCspDebug,          // handle to debug text file
                        szCspDebug,         // buffer of data to write to file
                        nCspDebugLength,    // number of bytes to write
                        &nCspDebugLength,   // number of bytes written
                        0);                 // no overlapped structure present
            }
    #endif
 
// verify CRC 
	aByteBuffer[i]=(char) csp2Getc();
	aByteBuffer[i+1] =(char) csp2Getc();

	if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
	{
		#ifdef _DEBUG
        if (bCspDebugMode)
		{
            nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
				"\r\n    CRCFailure:");
            WriteFile(  hCspDebug,          // handle to debug text file
				szCspDebug,         // buffer of data to write to file
				nCspDebugLength,    // number of bytes to write
				&nCspDebugLength,   // number of bytes written
				0);                 // no overlapped structure present
			for (j=0;j<(i+2);j++) {
				nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
					" %02X",(unsigned char) aByteBuffer[j]);
				WriteFile(  hCspDebug,          // handle to debug text file
					szCspDebug,         // buffer of data to write to file
					nCspDebugLength,    // number of bytes to write
					&nCspDebugLength,   // number of bytes written
					0);                 // no overlapped structure present
			}
		}
		#endif
		
		return( COMMAND_LRC_ERROR );
	}
	

    return( STATUS_OK );
}

/*********************************************************************                      
* Synopsis:       long DLL_IMPORT_EXPORT  csp2Interrogate(void)
*
* Description:    Request a response from the CSP device indicating that it is 
*                 operating and containing version information. Valid response 
*                 information is saved in local storage.
*
*                 The interrograte function will automatically retry the 
*                 interrogation operation up to nCspRetryCount times before 
*                 aborting.
*
* Parameters:     None  
*
* Return Value:   STATUS_OK
*                 COMMUNICATIONS_ERROR
*                 INVALID_COMMAND_NUMBER
*                 COMMAND_LRC_ERROR
*                 RECEIVED_CHARACTER_ERROR
*                 GENERAL_ERROR
*
* Inputs:           
*
* Outputs:          
*
* External Calls:   
*
* Notes:            
**********************************************************************/
/*********************************************************************
* Pseudocode:     
*
**********************************************************************/
unsigned char aInterrogateCmd[] =
    {
    INTERROGATE,      // Interrogate Command
    STX,              // opcode
    0x00,             // NULL terminate the message
    0x9f,             // CRC HH
    0xde              // CRC LL
    };

NoMangle long DLL_IMPORT_EXPORT csp2Interrogate(void)
{
  
	long i, j;
    long nRetStatus;
    long nCount = nCspRetryCount;

    #ifdef _DEBUG
	struct _timeb now;
	#endif

    // interrogate the CSP device up to nCspRetryCount+1 times...
    do
        {
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                _strtime( szDebugTime );        // Get time stamp string
				_ftime(&now);
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n\r\nInterrogate: [Time: %s %d.%d]\r\n", 
                                            szDebugTime,now.time,now.millitm);
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif

        // invalidate the old interrogation status...
        nCspDeviceStatus               = -1;   // no current Device status
        nCspProtocolVersion         = -1;   // no protocol version available
        nCspSystemStatus            = -1;   // no system status available
        szCspDeviceId[0]            =  0;   // no user Id available
        szCspSwVersion[0]           =  0;   // no software version available

        // make sure that a comm port has been selected...
        if (nCspActivePort < COM1)
            {
            #ifdef _DEBUG
                if (bCspDebugMode)
                    {
                    nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                "\r\nNo valid COM port selected; Interrogation aborted.\r\n");
                    WriteFile(  hCspDebug,          // handle to debug text file
                                szCspDebug,         // buffer of data to write to file
                                nCspDebugLength,    // number of bytes to write
                                &nCspDebugLength,   // number of bytes written
                                0);                 // no overlapped structure present
                    }
            #endif

            return( COMMUNICATIONS_ERROR );
            }

        // send the command to the CSP device...
        nRetStatus = csp2SendCommand ((char *)aInterrogateCmd, sizeof(aInterrogateCmd));

        // if the Device reported an error, try again from the top...
        if (nRetStatus != STATUS_OK)
            {
			nDelayCount=nDelayCount+2; // add 100 ms to delay time
            #ifdef _DEBUG
                if (bCspDebugMode)
                    {
                    nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                                "\r\nComm Delay now %d ms.\r\n",(nDelayCount*50));
                    WriteFile(  hCspDebug,          // handle to debug text file
                                szCspDebug,         // buffer of data to write to file
                                nCspDebugLength,    // number of bytes to write
                                &nCspDebugLength,   // number of bytes written
                                0);                 // no overlapped structure present
                    }
            #endif
            continue;
            }

        // Response looks good, get the entire message...
        i = 1;

        // get the STX character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    STX:              ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        aByteBuffer[i++] = (char) csp2Getc();

        // get the protocol version character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    Protocol Version: ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        nCspProtocolVersion = csp2Getc();
        aByteBuffer[i++] = (char) nCspProtocolVersion;

        // get the system status character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    System Status:    ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        nCspSystemStatus = csp2Getc();
        aByteBuffer[i++] = (char) nCspSystemStatus;

        // get the user id characters...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n  Device ID:          ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        for ( j = 0; j < 8 ; j++ )
            aByteBuffer[i++] = szCspDeviceId[j] = (char) csp2Getc();

        // get the software version characters...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    Software Version: ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        for ( j = 0; j < 8 ; j++ )
            aByteBuffer[i++] = szCspSwVersion[j] = (char) csp2Getc();

        // get the NULL character...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    NULL:             ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif
        aByteBuffer[i++] = (char) csp2Getc();

        // verify the CRC...
        #ifdef _DEBUG
            if (bCspDebugMode)
                {
                nCspDebugLength = wsprintf( (LPTSTR) szCspDebug, 
                                            "\r\n    CRC:              ");
                WriteFile(  hCspDebug,          // handle to debug text file
                            szCspDebug,         // buffer of data to write to file
                            nCspDebugLength,    // number of bytes to write
                            &nCspDebugLength,   // number of bytes written
                            0);                 // no overlapped structure present
                }
        #endif

// verify CRC 
	    aByteBuffer[i]=(char) csp2Getc();
	    aByteBuffer[i+1] =(char) csp2Getc();

		if (VerifyCRC((unsigned char *)aByteBuffer, i) != TRUE)
			{
		    nRetStatus = COMMAND_LRC_ERROR;
			continue;
			}


        // CSP device was interrogated successfully...
        return( STATUS_OK );

        } while (nCount--);

    return (nRetStatus);            // interrogation failed
}

/*******************************************************************
 *                       End of file                               *
 *******************************************************************/
