/*
// TestConsole.cpp : Defines the entry point for the console application.
//
*/

#include <windows.h>	/* needed to define FARPROC in CSP2.H */
#include <stdio.h>
#include "csp32.h"


#define DefaultCommPort COM6

unsigned char BarcodeIDByte[]={'A','B','D','K','F','G','N'};
unsigned char BarcodeTypes[(sizeof(BarcodeIDByte)/sizeof(unsigned char))+1][20]=
								{"UPC/EAN",
								 "Code39",
								 "EAN128",
								 "Code128",
								 "I 2of 5",
								 "D 2of 5",
								 "Coupon Code",
								 "Unknown"};

int main(int argc, char** argv)
{
	long    nRetStatus;
	long	PacketLength,BarcodesRead;
	char	Packet[64],aBuffer[256];
	int		i,k,Port;


	Port=DefaultCommPort;
	if (argc>1)
	{
		Port=((int)argv[1][0])-'1';
		if ((Port>15) || (Port<0)) Port = DefaultCommPort;
	}


	printf("Using comm %d\n",(Port+1));

	cspSetDebugMode(PARAM_ON);

	nRetStatus = cspGetDllVersion(aBuffer,256);
	if (nRetStatus!= STATUS_OK)
	{
		printf("No DLL Version Inf Available\r\n");
	}
	else
	{
		printf("Dll Version %s\n",aBuffer);
	}
	

    if ((nRetStatus = cspInit(Port)) != STATUS_OK) // Connected to COM x
	{
		/* error condition, alert the user of the problem */
		printf( "Comm port not initialized.\r\nError Status:%d", 
			nRetStatus);
		return 1;
	}   
	

	/* Get Device Id */
	nRetStatus = cspGetDeviceId(aBuffer, 8);
	printf("Device ID ");
	for (k=0;k<nRetStatus;k++) 
		printf(" %02X",(unsigned char) aBuffer[k]);
	
	/* Get Device Software Revision */
	nRetStatus = cspGetSwVersion(aBuffer, 9);
	printf("\nDevice SW Version: %s\n", aBuffer);
	
	nRetStatus = cspReadData(); /* Read barcodes */
	if (nRetStatus<0) 
	{
		/* error condition, alert the user of the problem */
		printf( "Error reading barcode info.\r\nError Status: %d", 
			nRetStatus);
	}
	
	else 
	{
		/* save number of barcodes read into DLL */
		BarcodesRead = nRetStatus;	
		printf("Read %d packets\n", BarcodesRead); 


		for (i=0;i<BarcodesRead;i++) 
		{
			PacketLength = cspGetBarcode(Packet,i,63); /* Read packets */
			for (k=0;k<(sizeof(BarcodeIDByte)/sizeof(unsigned char));k++) // get type
			{
				if (Packet[0]==BarcodeIDByte[k]) break;
			}
			printf("\nBarcode %d Type %s %s\n", i+1,BarcodeTypes[k],Packet+1); 			
			/* print out packet (hex) */
			for (k=0;k<PacketLength;k++) printf(" %02X",(unsigned char) Packet[k]);
			printf("\n");
		} /* end of read packets loop */
	}
	nRetStatus = cspPowerDown();
	printf("PowerDown response %d\n",nRetStatus);
	printf("\nPress return to exit");
	getchar(); /* wait for keyboard to exit */
	return 0;
}


